
#include <algorithm>
#include "GL/glui.h"
#include "tentacle.h"

Tentacle::Tentacle(float px, float py, float w, float h, float a, Tentacle *p)
	: positionX(px)
	, positionY(py)
	, width(w)
	, height(h)
	, angle(a)
	, r(.741f), g(.196f), b(.776f)
	, selected(false)
	, parent(p)
	, children()
{
    if(parent != NULL)
    {
        parent->addChild(this);
    }
}

Tentacle::~Tentacle()
{
	for(TVIterator i = children.begin(); i != children.end(); ++i)
	{
		delete *i;
	}
}

void Tentacle::setupUI(GLUI *parent_win)
{
	GLUI_Spinner *pxSpin = parent_win->add_spinner("Pos X"
										, GLUI_SPINNER_FLOAT
										, &positionX
			);

	GLUI_Spinner *pySpin = parent_win->add_spinner("Pos Y"
										, GLUI_SPINNER_FLOAT
										, &positionY
			);

	GLUI_Spinner *wSpin = parent_win->add_spinner("Width"
										, GLUI_SPINNER_FLOAT
										, &width
			);

	GLUI_Spinner *hSpin = parent_win->add_spinner("Height"
										, GLUI_SPINNER_FLOAT
										, &height
			);

	GLUI_Spinner *angSpin = parent_win->add_spinner("Angle"
										, GLUI_SPINNER_FLOAT
										, &angle
			);

	GLUI_Spinner *rSpin = parent_win->add_spinner("Red"
										, GLUI_SPINNER_FLOAT
										, &r
			);

	GLUI_Spinner *gSpin = parent_win->add_spinner("Green"
										, GLUI_SPINNER_FLOAT
										, &g
			);

	GLUI_Spinner *bSpin = parent_win->add_spinner("Blue"
										, GLUI_SPINNER_FLOAT
										, &b
			);


    wSpin->set_float_limits(0.05f, 1000.0f);
    hSpin->set_float_limits(0.05f, 1000.0f);

    wSpin->set_speed(0.1f);
    hSpin->set_speed(0.1f);

	angSpin->set_float_limits(-180.0f, 180.0f, GLUI_LIMIT_WRAP);
	angSpin->set_speed(0.1f);

	rSpin->set_float_limits(0.0f, 1.0f);
	gSpin->set_float_limits(0.0f, 1.0f);
	bSpin->set_float_limits(0.0f, 1.0f);

	rSpin->set_speed(0.05f);
	gSpin->set_speed(0.05f);
	bSpin->set_speed(0.05f);
}

void Tentacle::addChild(Tentacle *child)
{
	children.push_back(child);
}

void Tentacle::setSelected(bool val)
{
	selected = val;
}

void Tentacle::glRender()
{
	glPushMatrix();

	glTranslatef(positionX, positionY, 0.0f);
	glRotatef(angle, 0.0f, 0.0f, 1.0f);

	glPushMatrix();
	glScalef(width, height, 1.0f);

	glColor3f(r, g, b);

	glBegin(GL_TRIANGLE_STRIP);
		glVertex2i(0, 0);
		glVertex2i(1, 0);
		glVertex2i(0, 1);
		glVertex2i(1, 1);
	glEnd();

	if(selected)
	{
		glColor3f(1.0f, 1.0f, 0.0f);
	}
	else
	{
	    glColor3f(0.435f, 0.114f, 0.463f);
	}

    glLineWidth(3.0);
    glBegin(GL_LINE_LOOP);
        glVertex2i(0, 0);
        glVertex2i(1, 0);
        glVertex2i(1, 1);
        glVertex2i(0, 1);
    glEnd();


	glPopMatrix();

	for(TVIterator i = children.begin(); i != children.end(); ++i)
	{
		(*i)->glRender();
	}

	glPopMatrix();
}

Tentacle *Tentacle::getParent()
{
	return parent;
}

Tentacle *Tentacle::nextSibling()
{
	if(parent == NULL)
	{
		return this;
	}
	else
	{
		return parent->nextChild(this);
	}
}

Tentacle *Tentacle::firstChild()
{
	if(children.size() > 0)
	{
		return children[0];
	}
	else
	{
		return NULL;
	}
}

Tentacle *Tentacle::nextChild(Tentacle *child)
{
	TVIterator pos = std::find(children.begin(), children.end(), child);

	if(pos == children.end())
	{
		// BIG PROBLEM HERE!!
	}

	++pos;

	if(pos == children.end())
	{
		pos = children.begin();
	}

	return *pos;
}
